import {Component, OnInit} from '@angular/core';
import {ApiService} from '../../services/api.service';
import {UtilsService} from '../../services/utils.service';
import {Event} from '../../models/event';
import {Session} from '../../models/session';
import {User} from '../../models/user';

@Component({
	selector: 'app-admin',
	templateUrl: './admin.component.html',
	styleUrls: ['./admin.component.css']
})
export class AdminComponent implements OnInit {

	isLoggedIn: boolean = false;
	events: Event[] = [];
	selectedCalendar: string = '';
	password: string = '';
	name: string = '';
	email: string = '';
	eventFilter: string = 'future'; // Default value for filter
	eventSorting: string = 'start_asc'; // Default value for sorting
	isActive: boolean = false;
	registerEmail: string = '';
	registerPassword: string = '';
	registerPasswordConfirm: string = '';

	constructor(
		private api: ApiService
	) {
	}

	ngOnInit(): void {
		if (UtilsService.getSessionInfoFromLocalStorage().sessionId !== -1) {
			this.api.checkSession(UtilsService.getSessionInfoFromLocalStorage()).subscribe((user: User) => {
				if(user.userId != null && user.userId !== -1) {
					this.isLoggedIn = true;
					this.name = user.fullName;
					this.isActive = user.isActive;
					this.getEvents();
				}
			});
		}
	}

	getEvents(): void {
		this.events = [];

		if (this.selectedCalendar === '') {
			return;
		}

		this.api.getEvents(this.selectedCalendar).subscribe((events: Event[]): void => {
			for (let event of events) {
				if(event.status !== 'DELETED') {
					this.events.push({
						...event,
						startDateTime: new Date(event.startDateTime),
						endDateTime: new Date(event.endDateTime),
						createdDate: new Date(event.createdDate)
					});
				}
			}
			this.filterEvents();
			this.sortEvents();
		});
	}

	handleCalendarChange() {
		this.getEvents();
	}

	getCalendarId(text: string): number {
		switch (text) {
			case 'public':
				return 1;
			case 'members':
				return 2;
			case 'choir':
				return 4;
			case 'management':
				return 3;
			default:
				return -1;
		}
	}

	filterEvents() {
		this.events = this.events.filter((event) => {
			return this.checkEventMeetsFilterCriteria(event);
		});
	}

	checkEventMeetsFilterCriteria(event: Event): boolean {
		switch (this.eventFilter) {
			case '':
				return true;
			case 'all':
				return true;
			case 'future':
				return this.reduceToDay(new Date(event.endDateTime)) >= this.reduceToDay(new Date());
			case 'past':
				return this.reduceToDay(new Date(event.endDateTime)) < this.reduceToDay(new Date());
			default:
				return true;
		}
	}

	reduceToDay(date: Date): Date {
		let dayOnlyDate: Date = new Date();
		dayOnlyDate.setFullYear(date.getFullYear());
		dayOnlyDate.setMonth(date.getMonth());
		dayOnlyDate.setDate(date.getDate());

		return dayOnlyDate;
	}

	getUserName(): string {
		return this.name;
	}

	sortEvents(): void {
		this.events.sort((a, b) => {
			switch (this.eventSorting) {
				case '':
					return 1;
				case 'start_asc':
					return a.startDateTime > b.startDateTime ? 1 : -1;
				case 'start_desc':
					return a.startDateTime > b.startDateTime ? -1 : 1;
				case 'created_asc':
					return a.createdDate > b.createdDate ? 1 : -1;
				case 'created_desc':
					return a.createdDate > b.createdDate ? -1 : 1;
				default:
					return 1;
			}
		});
	}

	login(): void {
		this.api.login(this.email, this.password).subscribe((session: Session): void => {
			if(session.sessionId != null && session.sessionId !== -1) {
				UtilsService.saveSessionInfoToLocalStorage(session.sessionId, session.sessionKey);

				// Get user info
				this.api.checkSession(UtilsService.getSessionInfoFromLocalStorage()).subscribe((user: User) => {
					if(user.userId != null && user.userId !== -1) {
						this.isLoggedIn = true;
						this.name = user.fullName;
						this.isActive = user.isActive;
						this.getEvents();
					} else {
						confirm('Login unsuccessful. Please check if you provided the correct username and password.');
					}
				});
			}
		}, (error) => {
			confirm('Login unsuccessful. Reported problem from server: ' + error.error.message);
		});
	}

	register(): void {
		this.api.register(this.registerEmail, this.name, this.registerPassword).subscribe((session: Session): void => {
			if(session.sessionId != null && session.sessionId !== -1) {
				UtilsService.saveSessionInfoToLocalStorage(session.sessionId, session.sessionKey);
				this.isLoggedIn = true;
				this.getEvents();
				confirm('An email was sent to your Nachklang address. Please click the link in the email to activate your account. You can\'t use this application before the activation.');
			} else {
				confirm('Regisistration unsuccessful. Please contact Patrick.');
			}
		}, (error) => {
			confirm('Login unsuccessful. Reported problem from server: ' + error.error.message);
		});
	}

	logout(): void {
		UtilsService.clearSessionInfo();
		this.isLoggedIn = false;
	}

	checkUserInactive(): boolean {
		return !this.isActive;
	}

	checkPasswordsMatch(): boolean {
		return this.registerPassword === this.registerPasswordConfirm;
	}

	checkPasswordPolicy(): boolean {
		let isLongEnough = this.registerPassword.length >= 12;

		var lowercaseRegex = /[a-z]/g
		let hasLowercase = lowercaseRegex.test(this.registerPassword);

		var uppercaseRegex = /[A-Z]/g
		let hasUppercase = uppercaseRegex.test(this.registerPassword);

		var numberRegex = /[0-9]/g
		let hasNumbers = numberRegex.test(this.registerPassword);

		return isLongEnough && hasLowercase && hasUppercase && hasNumbers;
	}
}
